@@ -22,6 +22,7 @@ def _open(filename):
 
 
 ENGINE_PREFIX = re.compile(r'^\s*([a-z\-_]+):\s*')
+STRUCTURE_PREFIX = re.compile(r'\s*(structure|text)\s+(.*)', re.DOTALL)
 WHITESPACE = re.compile(u"\s+")
 EXPRESSION = re.compile(u"\s*\${(.*?)}\s*")
 UNDERSCORE_CALL = re.compile("_\(.*\)")
@@ -165,16 +166,26 @@ class Extractor(ElementProgram):
             if attribute[1] in ['content', 'replace']:
                 (engine, value) = get_tales_engine(value)
                 if engine == 'python':
+                    m = STRUCTURE_PREFIX.match(value)
+                    if m is not None:
+                        value = m.group(2)
+                    value = '(%s)' % value
                     self._assert_valid_python(value)
                     yield value
             if attribute[1] == 'define':
                 for (scope, var, value) in parse_defines(value):
                     (engine, value) = get_tales_engine(value)
                     if engine == 'python':
+                        value = '(%s)' % value
                         self._assert_valid_python(value)
                         yield value
             elif attribute[1] == 'repeat':
-                (engine, value) = get_tales_engine(value.split(None, 1)[1])
+                defines = parse_defines(value)
+                if len(defines) != 1:
+                    print('Aborting due to syntax error in %s[%d]: %s' % (
+                            self.filename, self.linenumber, value))
+                scope, var, value = defines[0]
+                (engine, value) = get_tales_engine(value)
                 if engine == 'python':
                     self._assert_valid_python(value)
                     yield value
@@ -188,8 +199,7 @@ class Extractor(ElementProgram):
                 sys.exit(1)
 
     def parse_python(self, source):
-        if not isinstance(source, bytes):
-            source = source.encode('utf-8')
+        assert isinstance(source, type(u''))
         for message in _extract_python(self.filename, source, self.options, self.linenumber):
             self.messages.append(Message(*message[:6],
                 location=(self.filename, self.linenumber + message.location[1])))
